pro darkvals,cubelist,rawlist

; cubelist is a list of fits cubes
; rawlist is a matching list of the raw frames

spawn,'wc '+cubelist,tmpfd
reads,tmpfd,nlines,tmpfd1
cubeframes = strarr(nlines)
openr,1,cubelist
readf,1,cubeframes
close,1

spawn,'wc '+rawlist,tmpfd
reads,tmpfd,nlines,tmpfd1
rawframes = strarr(nlines)
openr,1,rawlist
readf,1,rawframes
close,1

for ilist=0,n_elements(cubeframes)-1 do begin
print,'processing frame ',ilist+1
   cubename = cubeframes[ilist]
   rawname = rawframes[ilist]

;read in object & reference data
obj = fltarr(14,14,24) + !values.f_nan
drk = fltarr(14,14,24) + !values.f_nan
darkname = 'darktrace_image.fits'
for xtn=1,24 do begin
   fits_read,cubename,ocube,hdr,exten_no=xtn,/no_pdu
   if (n_elements(ocube) gt 1) then begin
      for ix=0,13 do begin
         for iy=0,13 do begin
            vec = ocube[ix,iy,*]
            obj[ix,iy,xtn-1] = median(vec[where(finite(vec))])
         endfor
      endfor
   endif
   fits_read,darkname,d,hdr,exten_no=xtn,/no_pdu
   drk[*,*,xtn-1] = d
endfor

;estimate residual channel values
chanval = fltarr(32*3)
;objout = obj
for i=1,32*3 do begin
   rge = where(drk gt i-0.1 and drk lt i+0.1)
   tmp1 = obj[rge]
   tmp2 = where(finite(tmp1),tmpi)
   med2 = 0.
   if (tmpi gt 0) then begin
      vals = tmp1[tmp2]
      med1 = median(vals)                   ; 50%ile
      med2 = median(vals[where(vals le med1)]) ; 25%ile
   endif
   chanval[i-1] = med2
   rge2 = where(drk gt i-0.5 and drk lt i+0.5)
   ;objout[rge2] = obj[rge2] - med2
;if (i eq 5) then stop
print,i,n_elements(rge),med2,n_elements(rge2)
endfor
;writefits,'sub_'+imagename,objout
;writefits,'orig_'+imagename,obj

;stop

;plot,chanval,psym=3
cvtmp = chanval
cvtmpnan = cvtmp
cvtmpnan[where(cvtmp eq 0)] = !values.f_nan
;oplot,cvtmp,psym=2
plot,[0,96],[-10,10],/nodata
for iv = 0,2 do begin
cv = chanval[iv*32:iv*32+31]
cvf = cv[where(cv ne 0)]
med = median(cvf)
sdv = stddev(cvf)
cvf = cvf[where(cvf le med+3*sdv and cvf ge med-3*sdv)]
med = median(cvf)
sdv = stddev(cvf)
;oplot,[iv*32,iv*32+31],[med,med]
;oplot,[iv*32,iv*32+31],[med,med]-3*sdv,linestyle=2
;oplot,[iv*32,iv*32+31],[med,med]+3*sdv,linestyle=2
nbad = where((cv gt med+3*sdv or cv lt med-3*sdv) and cv ne 0,nbadi)
if (nbadi gt 0) then begin
   print,'rejected '+string(nbadi)+' values from detector'+string(iv+1)
   cv[nbad] = med
   chanval[nbad+iv*32] = med
endif
chanval[iv*32:iv*32+31] = chanval[iv*32:iv*32+31] - med
oplot,indgen(32)+iv*32,cvtmp[iv*32:iv*32+31]-med,psym=1
oplot,indgen(32)+iv*32,cvtmpnan[iv*32:iv*32+31]-med,psym=2
oplot,[iv*32,iv*32+31],[0,0]
oplot,[iv*32,iv*32+31],[0,0]-3*sdv,linestyle=2
oplot,[iv*32,iv*32+31],[0,0]+3*sdv,linestyle=2
endfor
wait,0.1

;stop

;subtract channel values from raw frame
fname = 'chan_'+rawname
spawn,'cp '+rawname+' '+fname

d = fltarr(2048,2048)

for xtn=1,3 do begin
   for i=0,31 do begin
      d[i*64:i*64+63,*] = chanval[i+32*(xtn-1)]
      ;print,xtn,i,(i+1)+32*(xtn-1),chanval[i+32*(xtn-1)]
   endfor
   fits_read,rawname,oraw,hdr,exten_no=xtn,/no_pdu
   modfits,fname,oraw-d,hdr,exten_no=xtn
endfor


endfor

end
